#!/usr/bin/env perl
##
##  OSSP ase -- Affiliation Service Environment
##  Copyright (c) 2005 Ralf S. Engelschall <rse@engelschall.com>
##  Copyright (c) 2005 The OSSP Project <http://www.ossp.org/>
##
##  This file is part of OSSP ase, a service environment for managing
##  affiliations which can be found at http://www.ossp.org/pkg/tool/ase/.
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
##  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
##
##  ase.cgi: application main procedure
##

#   language requirements
require 5.008;
use strict;
use warnings;

#   program configuration
our $cfg = {
    -prog_name   => 'OSSP ase',
    -prog_desc   => 'Affiliation Service Environment',
    -prog_vers   => '0.0.1',
    -prog_date   => '20-Feb-2005',
    -path_libdir => '.',
    -path_vardir => '.',
    -path_datdir => '.',
};

#   language extensions
use Socket;                             # from OpenPKG "perl"
use IO::All;                            # from OpenPKG "perl-sys"
use OSSP::uuid;                         # from OpenPKG "uuid" (with_perl=yes)
use GD;                                 # from OpenPKG "perl-gd"
use DBI;                                # from OpenPKG "perl-dbi"
use DBD::SQLite;                        # from OpenPKG "perl-dbi" (with_dbd_sqlite=yes)
use DB_File;                            # from OpenPKG "perl-db"
use Data::Dumper;                       # from OpenPKG "perl"
use Storable;                           # from OpenPKG "perl"
use String::Divert;                     # from OpenPKG "perl-util"
use Locale::Language;                   # from OpenPKG "perl-locale"
use Locale::Country;                    # from OpenPKG "perl-locale"
use Crypt::UnixCrypt;                   # from OpenPKG "perl-crypto"
use Crypt::PasswdMD5;                   # from OpenPKG "perl-crypto"
use Digest::MD5;                        # from OpenPKG "perl-crypto"
use Digest::SHA1;                       # from OpenPKG "perl-crypto"
use Mail::Sendmail;                     # from OpenPKG "perl-mail"
use HTML::Entities qw();                # from OpenPKG "perl-www"
use URI::Escape;                        # from OpenPKG "perl-www"
use CGI;                                # from OpenPKG "perl-www"
use CGI::Cookie;                        # from OpenPKG "perl-www"
use CGI::Application;                   # from OpenPKG "perl-www"
use CGI::Session;                       # from OpenPKG "perl-www"
use CGI::FormBuilder;                   # from OpenPKG "perl-www"
use CGI::GuruMeditation ("OSSP ase");   # from OpenPKG "perl-www"

#   application parts
my $dir = $cfg->{-path_libdir};
require "$dir/ase.cgi.main.pl";         # application main framework
require "$dir/ase.cgi.loader.pl";       # application file loader
require "$dir/ase.cgi.util.pl";         # application utility functions
require "$dir/ase.cgi.scode.pl";        # application security code functions
require "$dir/ase.cgi.login.pl";        # application workflow: login
require "$dir/ase.cgi.logout.pl";       # application workflow: logout
require "$dir/ase.cgi.enroll.pl";       # application workflow: enrollment
require "$dir/ase.cgi.profile.pl";      # application workflow: profile

#   main procedure
my $app = new ASE ($cfg);
$app->run();
exit(0);

