##
##  OSSP ase -- Affiliation Service Environment
##  Copyright (c) 2005 Ralf S. Engelschall <rse@engelschall.com>
##  Copyright (c) 2005 The OSSP Project <http://www.ossp.org/>
##
##  This file is part of OSSP ase, a service environment for managing
##  affiliations which can be found at http://www.ossp.org/pkg/tool/ase/.
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
##  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
##
##  ase.cgi.enroll.pl: enrollment
##

package ASE;
use base 'CGI::Application';
use strict;
use warnings;

#   application run-time mode processing: "enrollment"
sub mode_enrollment_screen {
    my $self = shift;
    my $cgi  = $self->param(-cgi);
    my $html = $self->param(-html);

    #   generate the outer canvas
    $html .= "<div class=\"boxed\">\n";
    $html .= "<span class=\"title\">Account Enrollment</span>\n";
    $html .=
        "<ul class=\"breadcrumb\">\n" .
        "  <li><a href=\"".$self->url(-mode => "login")."\">Login &rarr;</a></li>\n" .
        "  <li>Enrollment:</li>\n" .
        "</ul>\n";
    $html .= $cgi->start_form(
        -method => 'POST',
        -action => $self->url(),
        -class  => 'enrollment',
    ) . "\n";
    $html .= $cgi->hidden(
        -name     => 'mode',
        -default  => 'enrollment-action',
        -override => 1
    ) . "\n";
    $html->fold("canvas");
    $html .= $cgi->end_form() . "\n";
    $html .= "</div>\n";
    $html >> "canvas";

    #   generate step-by-step list
    $html .=
        "<ol class=steps>\n" .
        "  <li class=this>Account Specification</li>\n" .
        "  <li>Account Verification</li>\n" .
        "  <li>Account Activation</li>\n" .
        "</ol>\n";

    #   generate description
    $html .=
        "<p>\n" .
        "Enroll here and join the OpenPKG Foundation e.V.!<p>\n" .
        "The enrollment as a <i>Fellow</i> of the OpenPKG Foundation e.V. is <i>free of charge</i>.<br>\n" .
        "Later you can even upgrade your status to a full <i>Member</i>\n" .
        "of the OpenPKG Foundation, too.<p>\n" .
        "For successfully enrolling you have to specify a valid\n" .
        "\"Email Address\" and a \"Password\". Additionally, you have to\n" .
        "repeat the \"Security Code\" for proving you are human.\n" .
        "After pressing \"Enroll\"\n" .
        "you will receive a verification Email under the specified\n" .
        "address containing an activation URL. Once you clicked on\n" .
        "this URL, you are successfully enrolled.\n";

    #   determine some information
    my $pw = $self->makerandom(8, "A-Za-z0-9");
    my $remote = $self->remotehostname();

    #   generate the inner canvas
    $html .=
        "<p>\n" .
        "<table class=\"F\">\n" .
        "  <tr>\n" .
        "    <td class=\"L\"><span class=\"label\">Email Address:</span></td>\n" .
        "    <td class=\"I\">"; $html->fold("field-address"); $html .= "</td>\n" .
        "  </tr>\n" .
        "  <tr>\n" .
        "    <td class=\"L\"><span class=\"label\">Password:</span></td>\n" .
        "    <td class=\"I\">"; $html->fold("field-password"); $html .= "</td>\n" .
        "  </tr>\n" .
        "  <tr>\n" .
        "    <td class=\"L\"><span class=\"label\">Security Code:</span></td>\n" .
        "    <td class=\"I\">"; $html->fold("field-scode"); $html .= "</td>\n" .
        "  </tr>\n" .
        "  <tr>\n" .
        "    <td>&nbsp;</td>\n" .
        "    <td>"; $html->fold("button-enroll"); $html .= "</td>\n" .
        "  </tr>\n" .
        "</table>\n";

    #   generate a security code
    my $scode_uuid = ($cgi->param("scode.uuid") || $self->scode_create());
    $html .= $cgi->hidden(
        -name     => 'scode.uuid',
        -value    => $scode_uuid,
        -override => 1
    ) . "\n";

    #   generate the input fields
    $html >> "field-address";
    $html .= $cgi->textfield(
        -id        => 'field_address',
        -class     => 'field',
        -name      => 'address',
        -default   => '',
        -size      => 40,
        -maxlength => 80,
    ) . "<br>";
    $html .= "<span class=\"annotation\">(<i>name</i>\@<i>domain</i>, e.g. \"foo\@$remote\")</span>\n";
    if (defined(my $error = $cgi->param("address.error"))) {
        $html .= "<br><span class=\"error\">$error</span>\n";
    }
    $html << 1;
    $html >> "field-password";
    $html .= $cgi->password_field(
        -id        => 'field_password',
        -class     => 'field',
        -name      => 'password',
        -default   => '',
        -size      => 22,
        -maxlength => 20,
    ) . "<br>";
    $html .= "<span class=\"annotation\">(up to 20 characters, e.g. \"$pw\")</span>\n";
    if (defined(my $error = $cgi->param("password.error"))) {
        $html .= "<br><span class=\"error\">$error</span>\n";
    }
    $html << 1;
    $html >> "field-scode";
    $html .= "<img align=top src=\"".$self->url(-mode => 'scode', -uuid => $scode_uuid)."\"> :\n";
    $html .= $cgi->textfield(
        -id        => 'field_scode',
        -class     => 'field',
        -name      => 'scode',
        -default   => '',
        -size      => 7,
        -maxlength => 6,
    ) . "<br>";
    $html .= "<span class=\"annotation\">(repeat the shown 6 character security code)</span>\n";
    if (defined(my $error = $cgi->param("scode.error"))) {
        $html .= "<br><span class=\"error\">$error</span>\n";
    }
    $html << 1;
    $html >> "button-enroll";
    $html .= $cgi->submit(
        -class   => 'button',
        -id      => 'button_cancel',
        -name    => 'button.cancel',
        -value   => 'Cancel',
    );
    $html .= $cgi->submit(
        -class   => 'button',
        -id      => 'button_enroll',
        -name    => 'button.enroll',
        -value   => 'Enroll',
    );
    $html << 1;

    return;
}

#   run-time mode screen action: "enrollment"
sub mode_enrollment_action {
    my $self = shift;
    my $cgi  = $self->param(-cgi);
    my $html = $self->param(-html);
    my $dbi  = $self->param(-dbi);

    #   on-the-fly remove expired entries from database
    $dbi->do(
        "DELETE FROM ase_enrollment" .
        " WHERE en_expiry < ".$self->datetime().";"
    ) or die $dbi->errmsg;;

    #   make sure we are called correctly
    if (defined($cgi->param("button.cancel"))) {
        return $self->switchto("login");
    }
    elsif (not defined($cgi->param("button.enroll"))) {
        die "unable to determine action";
    }

    #   determine form parameters
    my $address    = $cgi->param("address");
    my $password   = $cgi->param("password");
    my $scode_num  = $cgi->param("scode");
    my $scode_uuid = $cgi->param("scode.uuid");

    #   verify form parameters
    my $error = 0;
    if ($address !~ m/^[^@]+\@(?:[^.]+\.)+[^.]+$/) {
        $cgi->param('address.error', 'Invalid Email address: not <i>name</i>@<i>domain</i>');
        $error = 1;
    }
    if ($password !~ m/^[a-zA-Z0-9]{6,}$/) {
        $cgi->param('password.error', 'Invalid Password: at least 6 characters required');
        $error = 1;
    }
    if (not $self->scode_verify($scode_uuid, $scode_num)) {
        $cgi->param('scode.error', 'Invalid Security Code');
        $error = 1;
    }

    #   make sure the Email address is still not used
    if (not defined($cgi->param('address.error'))) {
        my $sth = $dbi->prepare(
            "SELECT * FROM ase_enrollment WHERE en_address = ?;"
        ) or die $dbi->errstr();
        $sth->execute($address) or die $dbi->errstr();
        my $rec = $sth->fetchrow_hashref();
        if (defined($rec)) {
            $cgi->param('address.error', "Email address already in use for enrolling since " .
                $self->datetime2str($rec->{'en_start'}));
            $error = 1;
        }
        $sth = $dbi->prepare(
            "SELECT * FROM ase_member WHERE me_address = ?;"
        ) or die $dbi->errstr();
        $sth->execute($address) or die $dbi->errstr();
        $rec = $sth->fetchrow_hashref();
        if (defined($rec)) {
            $cgi->param('address.error', "Email address already in use since " .
                $self->datetime2str($rec->{'me_membership_start'}));
            $error = 1;
        }
    }

    #   redirect to enrollment screen if errors occurred
    if ($error) {
        return $self->switchto("enrollment");
    }

    #   generate membership UUID
    my $uuid = new OSSP::uuid;
    $uuid->make("v1");
    my $me_uuid = $uuid->export("str");
    undef $uuid;

    #   generate encrypted password variants
    my $pw = $self->makepasswords($password);

    #   create membership record in database
    my $now = time();
    my $t_start  = $self->datetime($now);
    my $t_expiry = $self->datetime($now+1*60);
    my $sth = $dbi->prepare(
        "INSERT INTO ase_enrollment" .
        " (en_uuid," .
        "  en_address, en_pw_sha1, en_pw_md5, en_pw_cryptmd5, en_pw_cryptdes," .
        "  en_start, en_expiry)" .
        " VALUES (?, ?, ?, ?, ?, ?, ?, ?);"
    ) or die $dbi->errstr();
    $sth->execute(
        $me_uuid, $address,
        $pw->{-sha1}, $pw->{-md5}, $pw->{-cryptmd5}, $pw->{-cryptdes}, 
        $t_start, $t_expiry
    ) or die $dbi->errstr();

    #   send activation URL to Email address
    my $url = $cgi->url(-base => 1) . $cgi->url(-absolute => 1) . "?mode=activation;uuid=$me_uuid";
    my $mail =
        "Thank you for your interest in joining the OpenPKG Foundation.\n" .
        "To finish your enrollment you have to activate your membership\n" .
        "account by going to the following URL until ".$self->datetime2str($t_expiry)." UTC:\n" .
        "\n" .
        "$url\n" .
        "\n" .
        "-- \n" .
        "Affiliation Service Environment\n" .
        "OpenPKG Foundation e.V.\n" .
        "" . $cgi->url(-base => 1) . $cgi->url(-absolute => 1) . "\n" .
        "\n";
    $Mail::Sendmail::mailcfg{"smtp"} = [qw(localhost mail)];
    Mail::Sendmail::sendmail(
        To      => "rse\@engelschall.com\n", #$address,
        From    => "OpenPKG Foundation Affiliation Service Environment <ase\@localhost>\n", # FIXME
        Subject => "OpenPKG Foundation Membership Enrollment\n",
        Message => $mail,
    ) or die $Mail::Sendmail::error;

    #   generate the outer canvas
    $html .= "<div class=\"boxed\">\n";
    $html .= "<span class=\"title\">Account Enrollment</span>\n";
    $html .=
        "<ul class=\"breadcrumb\">\n" .
        "  <li><a href=\"".$self->url(-mode => "login")."\">Login &rarr;</a></li>\n" .
        "  <li>Enrollment:</li>\n" .
        "</ul>\n";
    $html .= $cgi->start_form(
        -method => 'POST',
        -action => $self->url(),
        -class  => 'enrollment',
    ) . "\n";
    $html .= $cgi->hidden(
        -name     => 'mode',
        -default  => 'enrollment-action',
        -override => 1
    ) . "\n";
    $html->fold("canvas");
    $html .= $cgi->end_form() . "\n";
    $html .= "</div>\n";
    $html >> "canvas";

    #   back
    #$html .=
    #    "<a class=\"button\" href=\"".$self->url(-mode => "login")."\">" .
    #    "&larr; Back to <b>Login</b>" .
    #    "</a><p>\n";

    #   generate step-by-step list
    $html .=
        "<ol class=steps>\n" .
        "  <li>Account Specification</li>\n" .
        "  <li class=this>Account Verification</li>\n" .
        "  <li>Account Activation</li>\n" .
        "</ol>\n";

    #   generate header text
    $html .=
        "<p>\n" .
        "An Email was sent to you under &lt;<a href=\"mailto:$address\"><b>$address</b></a>&gt; containing\n" .
        "an URL. Please follow this URL for being directed to step 3\n" .
        "of this Enrollment procedure, the Account Activation.\n";

    return;
}

#   run-time mode screen rendering: "activation"
sub mode_activation_screen {
    my $self = shift;
    my $cgi  = $self->param(-cgi);
    my $dbi  = $self->param(-dbi);
    my $html = $self->param(-html);

    #   determine UUID
    my $uuid = $cgi->param("uuid");

    #   lookup enrollment information from database
    my $sth = $dbi->prepare(
        "SELECT * FROM ase_enrollment WHERE en_uuid = ?;"
    ) or die $dbi->errstr();
    $sth->execute($uuid) or die $dbi->errstr();
    my $rec = $sth->fetchrow_hashref();
    if (not defined($rec)) {
        $html .= "<div class=\"boxed\">\n" .
                 "<span class=\"title\">Account Enrollment</span>\n" .
                 "<h3 class=\"error\">ERROR: ACTIVATION FAILED</h3>\n" .
                 "Sorry, no enrollment information found under " .
                 "UUID $uuid. Either you waited too long for activating " .
                 "(the enrollment information already expired), or " .
                 "you already activated your account or " .
                 "the URL you followed was incorrect. Please start " .
                 "from scratch!" .
                 "<p>\n" .
                 "<a class=\"button\" href=\"".$self->url(-mode => "enrollment")."\">" .
                 "Go to <b>Enrollment</b> screen" .
                 "</a>\n" .
                 "</div>\n";
        return;
    }

    #   create new membership account
    my $now     = time();
    my $t_start = $self->datetime($now);
    my $t_end   = $self->datetime($now+365*24*60*60);
    $sth = $dbi->prepare(
        "INSERT INTO ase_member" .
        " (me_uuid, me_address, " .
        "  me_membership_status, me_membership_type, " .
        "  me_membership_start, me_membership_end, " .
        "  me_access_pw_sha1, me_access_pw_md5, me_access_pw_cryptmd5, me_access_pw_cryptdes)" .
        " VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?);"
    ) or die $dbi->errstr();
    $sth->execute(
        $uuid, $rec->{en_address},
        1, "N",
        $t_start, $t_end,
        $rec->{en_pw_sha1}, $rec->{en_pw_md5}, $rec->{en_pw_cryptmd5}, $rec->{en_pw_cryptdes},
    ) or die $dbi->errstr();

    #   delete enrollment information
    $dbi->do(
        "DELETE FROM ase_enrollment WHERE en_uuid = '$uuid';"
    ) or die $dbi->errmsg;;

    #   provide final information
    $html .=
         "<div class=\"boxed\">\n" .
         "<span class=\"title\">Account Enrollment</span>\n";
    $html .=
        "<ul class=\"breadcrumb\">\n" .
        "  <li><a href=\"".$self->url(-mode => "login")."\">Login &rarr;</a></li>\n" .
        "  <li>Enrollment:</li>\n" .
        "</ul>\n";
    $html->fold("canvas");
    $html .= "</div>\n";
    $html >> "canvas";

    #   generate step-by-step list
    $html .=
        "<ol class=steps>\n" .
        "  <li>Account Specification</li>\n" .
        "  <li>Account Verification</li>\n" .
        "  <li class=this>Account Activation</li>\n" .
        "</ol>\n";

    #   generate description
    $html .=
        "Congratulations!\n" .
        "<p>\n" .
        "You have successfully enrolled for a membership with the OpenPKG Foundation e.V.\n" .
        "Your current membership status is <b>Fellow</b>. If you want to become even a <b>Member</b>\n" .
        "of the OpenPKG Foundation e.V., please visit your account information and upgrade.</b>\n" .
        "You can now access your account by going to the Login screen.\n" .
        "<p>\n" .
        "<a class=\"button\" href=\"".$self->url(-mode => "login")."\">" .
        "Go to <b>Login</b> screen" .
        "</a>\n";

    return;
}

1;

