##
##  OSSP ase -- Affiliation Service Environment
##  Copyright (c) 2005 Ralf S. Engelschall <rse@engelschall.com>
##  Copyright (c) 2005 The OSSP Project <http://www.ossp.org/>
##
##  This file is part of OSSP ase, a service environment for managing
##  affiliations which can be found at http://www.ossp.org/pkg/tool/ase/.
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
##  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
##
##  ase.cgi.loader.pl: external file loader
##

package ASE;
use base 'CGI::Application';
use strict;
use warnings;

#   run-time mode external file loading: "loader"
sub mode_loader {
    my $self = shift;
    my $cfg  = $self->param(-cfg);
    my $cgi  = $self->param(-cgi);

    #   parameter sanity checking
    my $name = $cgi->param("name");
    die "invalid name" if (not (defined($name) and $name ne ''));
    die "invalid name" if ($name =~ m|\.\.| or $name =~ m|^/|); # FIXME

    #   load external file content 
    my $filename = $cfg->{-path_datdir}."/".$name;
    die "file not found" if (not -f $filename);
    my $content < IO::All::io($filename)
        or die "unable to load file \"$filename\"";

    #   determine expire time and content type
    my $expires = "+10s";
    my $type = "application/octet-stream";
    if    ($name =~ m|\.css$|)   { $type = "text/css";   $expires = "+1m"; }
    elsif ($name =~ m|\.png$|)   { $type = "image/png";  $expires = "+1h"; }
    elsif ($name =~ m|\.gif$|)   { $type = "image/gif";  $expires = "+1h"; }
    elsif ($name =~ m|\.jpe?g$|) { $type = "image/jpeg"; $expires = "+1h"; }

    #   optional content adjustment
    if ($name =~ m|\.css$|) {
        $content =~ s|url\((.+?)\)|"url(".$cgi->url(-relative => 1).
                                   "?mode=loader;name=".URI::Escape::uri_escape($1).")"|sge;
    }

    #   pass back content
    $self->header_props(-type => $type, -expires => $expires);
    return $content;
}

1;

