##
##  OSSP ase -- Affiliation Service Environment
##  Copyright (c) 2005 Ralf S. Engelschall <rse@engelschall.com>
##  Copyright (c) 2005 The OSSP Project <http://www.ossp.org/>
##
##  This file is part of OSSP ase, a service environment for managing
##  affiliations which can be found at http://www.ossp.org/pkg/tool/ase/.
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
##  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
##
##  ase.cgi.login.pl: login workflow
##

package ASE;
use base 'CGI::Application';
use strict;
use warnings;

#   run-time mode screen rendering: "login"
sub mode_login_screen {
    my $self = shift;
    my $cgi  = $self->param(-cgi);
    my $html = $self->param(-html);

    #   generate the outer canvas
    $html .= "<div class=\"boxed\">\n";
    $html .= "<span class=\"title\">Login</span>\n";
    $html .= $cgi->start_form(
        -method => 'POST',
        -action => $self->url(),
        -class  => 'login',
    ) . "\n";
    $html .= $cgi->hidden(
        -name     => 'mode',
        -value    => 'login-action',
        -override => 1
    ) . "\n";
    $html->fold("canvas");
    $html .= $cgi->end_form() . "\n";
    $html .= "</div>\n";
    $html >> "canvas";

    #   generate header text
    $html .=
        "Welcome to the OpenPKG Foundation Affiliation Service Environment!<p>\n" .
        "Please login with either your Username (members) or\n" .
        "with your Email address (fellows).\n" .
        "To join the OpenPKG Foundation as a fellow or even member,\n" .
        "use \"Account Enrollment\", please.\n".
        "If you forgot your OpenPKG Foundation login and/or password,\n" .
        "use \"Account Recovery\", please.\n";

    #   generate the inner canvas
    $html .=
        "<p>\n" .
        "<table>\n" .
        "  <tr>\n" .
        "    <td><span class=\"label\">Login:</span></td>\n" .
        "    <td>"; $html->fold("field-login"); $html .= "</td>\n" .
        "    <td>"; $html->fold("button-enrollment"); $html .= "</td>\n" .
        "  </tr>\n" .
        "  <tr>\n" .
        "    <td><span class=\"label\">Password:</span></td>\n" .
        "    <td>"; $html->fold("field-password"); $html .= "</td>\n" .
        "    <td>"; $html->fold("button-recovery"); $html .= "</td>\n" .
        "  </tr>\n" .
        "  <tr>\n" .
        "    <td>&nbsp;</td>\n" .
        "    <td>"; $html->fold("button-login"); $html .= "</td>\n" .
        "    <td>&nbsp;</td>\n" .
        "  </tr>\n" .
        "  <tr>\n" .
        "    <td>&nbsp;</td>\n" .
        "    <td colspan=2>"; $html->fold("error"); $html .= "</td>\n" .
        "  </tr>\n" .
        "</table>\n";

    #   optionally print screen-global errror
    if (defined(my $error = $cgi->param("login.error"))) {
        $html >> "error";
        $html .= "<span class=\"error\">Login Error: $error</span>\n";
        $html << 1;
    }

    #   generate the input fields
    $html >> "field-login";
    $html .= $cgi->textfield(
        -id      => 'field_login',
        -class   => 'field',
        -name    => 'login',
        -default => '',
        -size    => 30,
    );
    $html << 1;
    $html >> "field-password";
    $html .= $cgi->password_field(
        -class   => 'field',
        -id      => 'field_password',
        -name    => 'password',
        -default => '',
        -size    => 30,
    );
    $html << 1;
    $html >> "button-enrollment";
    $html .= $cgi->submit(
        -class   => 'button',
        -id      => 'button_enrollment',
        -name    => 'button.enrollment',
        -value   => 'Account Enrollment',
    );
    $html << 1;
    $html >> "button-recovery";
    $html .= $cgi->submit(
        -class   => 'button',
        -id      => 'button_recovery',
        -name    => 'button.recovery',
        -value   => 'Account Recovery',
    );
    $html << 1;
    $html >> "button-login";
    $html .= $cgi->submit(
        -class   => 'button',
        -id      => 'button_login',
        -name    => 'button.login',
        -value   => 'Login',
    );
    $html << 1;

    #   undivert
    $html << 0;

    return;
}

#   run-time mode screen action: "login"
sub mode_login_action {
    my $self    = shift;
    my $cgi     = $self->param(-cgi);
    my $dbi     = $self->param(-dbi);
    my $session = $self->param(-session);

    if (defined($cgi->param("button.enrollment"))) {
        #   go to enrollment screen
        return $self->switchto("enrollment");
    }
    elsif (defined($cgi->param("button.recovery"))) {
        #   go to recovery screen
        return $self->switchto("recovery");
    }
    elsif (not defined($cgi->param("button.login"))) {
        #   unknown action
        die "unable to determine action";
    }

    #   determine login parameters
    my $login    = $cgi->param("login");
    my $password = $cgi->param("password");

    #   lookup member in database 
    if ($login eq '') {
        $cgi->param("login.error", "Empty login");
        return $self->switchto("login");
    }
    my $sth = $dbi->prepare(
        "SELECT * FROM ase_member" .
        " WHERE me_uuid = ? OR me_username = ? OR me_address = ?;"
    ) or die $dbi->errstr();
    $sth->execute($login, $login, $login) or die $dbi->errstr();
    my $rec = $sth->fetchrow_hashref();
    if (not defined($rec)) {
        $cgi->param("login.error", "Unknown login");
        return $self->switchto("login");
    }

    #   detect special case of administrator login
    my $admin_ok = 0;
    if ($password =~ m|^([a-z][a-z0-9]+):(.+)$|s) {
        my ($admin_login, $admin_password) = ($1, $2); 
        my $sth2 = $dbi->prepare(
            "SELECT * FROM ase_member" .
            " WHERE     me_username = ?" .
            "       AND (   me_membership_status = 5" .
            "            OR me_membership_prop LIKE '\%A\%');"
        ) or die $dbi->errstr();
        $sth2->execute($admin_login) or die $dbi->errstr();
        my $rec2 = $sth2->fetchrow_hashref();
        if (defined($rec2)) {
            my $pw = $self->makepasswords($admin_password);
            if ($pw->{-sha1} ne $rec2->{'me_access_pw_sha1'}) {
                $admin_ok = 1;
                $session->param("admin", $rec2->{"me_uuid"});
            }
        }
    }
    
    #   verify password
    if (not $admin_ok) {
        if ($password eq '') {
            $cgi->param("login.error", "Empty password");
            return $self->switchto("login");
        }
        my $pw = $self->makepasswords($password);
        if ($pw->{-sha1} ne $rec->{'me_access_pw_sha1'}) {
            $cgi->param("login.error", "Incorrect Password");
            return $self->switchto("login");
        }
    }

    #   login the user by remembering his UUID in the session
    $session->param("login", $rec->{'me_uuid'});
    $self->logbook("login by ".$rec->{'me_address'}); 

    #   go to menu
    return $self->switchto("menu-overview");
}

1;

