##
##  OSSP ase -- Affiliation Service Environment
##  Copyright (c) 2005 Ralf S. Engelschall <rse@engelschall.com>
##  Copyright (c) 2005 The OSSP Project <http://www.ossp.org/>
##
##  This file is part of OSSP ase, a service environment for managing
##  affiliations which can be found at http://www.ossp.org/pkg/tool/ase/.
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
##  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
##
##  ase.cgi.logout.pl: logout workflow
##

package ASE;
use base 'CGI::Application';
use strict;
use warnings;

#   application run-time mode processing: "logout"
sub mode_logout_screen {
    my $self    = shift;
    my $cgi     = $self->param(-cgi);
    my $html    = $self->param(-html);
    my $session = $self->param(-session);

    #   remove the session from the database
    $session->delete();

    #   generate canvas
    $html .= "<div class=\"boxed\">\n";
    $html .= "<span class=\"title\">Logout</span>\n";
    $html->fold("canvas");
    $html .= "</div>\n";
    $html >> "canvas";

    #   generate description
    $html .=
        "Thank you for visiting the OpenPKG Foundation e.V. Affiliation Service Environment.<br>\n" .
        "Your session was terminated and you are now logged out.<br>\n" .
        "Please visit us again soon.";

    #   redirect to Login screen
    $html .=
        "<p>\n" .
        "You should be redirected to the Login screen in 10 seconds automatically.\n" .
        "<meta http-equiv=\"refresh\" content=\"10;url=".$self->url(-mode => "login")."\">\n" .
        "<p>\n" .
        "<a class=\"button\" href=\"".$self->url(-mode => "login")."\">" .
        "Go to <b>Login</b> screen" .
        "</a>\n";

    return;
}

#   application run-time mode processing: "menu-logout"
sub mode_menu_logout_screen {
    my $self    = shift;
    my $cgi     = $self->param(-cgi);
    my $html    = $self->param(-html);
    my $session = $self->param(-session);

    #   generate the outer canvas
    $html .= "<div class=\"boxed\">\n";
    $html .= "<span class=\"title\">Main Menu</span>\n";
    $html .= $cgi->start_form(
        -method => 'POST',
        -action => $self->url(),
        -class  => 'menu',
    ) . "\n";
    $html .= $cgi->hidden(
        -name     => 'mode',
        -value    => 'menu-logout-action',
        -override => 1
    ) . "\n";
    $html->fold("canvas");
    $html .= $cgi->end_form() . "\n";
    $html .= "</div>\n";
    $html >> "canvas";

    #   insert navigation tab
    $self->menu_tab("menu-logout");

    $html .=
        "Here you can logout from the OpenPKG Foundation Affiliation Service Environment. " .
        "Your current browser session will be destroyed and you will be redirected " .
        "to the Login screen again. Unless you explicitly logout, your session " .
        "will be kept active.";

    $html .=
        "<p>\n" .
        "<div align=center><a class=\"button\" href=\"".$self->url(-mode => "logout")."\">" .
        "<b>Logout!</b>" .
        "</a></div><p>\n";

    return;
}

1;

