##
##  OSSP ase -- Affiliation Service Environment
##  Copyright (c) 2005 Ralf S. Engelschall <rse@engelschall.com>
##  Copyright (c) 2005 The OSSP Project <http://www.ossp.org/>
##
##  This file is part of OSSP ase, a service environment for managing
##  affiliations which can be found at http://www.ossp.org/pkg/tool/ase/.
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
##  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
##
##  ase.cgi.main.pl: main program framework
##

package ASE;
use base 'CGI::Application';
use strict;
use warnings;

#   application init hook
sub cgiapp_init {
    my ($self, $cfg) = @_;

    #   establish application configuration structure
    $self->param(-cfg => $cfg);

    return;
}

#   application setup hook
sub setup {
    my ($self) = @_;
    my $cfg = $self->param(-cfg);

    #   establish CGI interface object
    my $cgi = new CGI;
    $self->query($cgi);
    $self->param(-cgi => $cgi);

    #   establish HTML output object
    my $html = new String::Divert;
    $html->folder('{#%s#}', '\{#([a-zA-Z_][a-zA-Z0-9_-]*)#\}');
    my $txt < IO::All::io($cfg->{-path_datdir}."/ase.html")
        or die "unable to read HTML skeleton file \"".$cfg->{-path_datdir}."/ase.html"."\"";
    $txt =~ s/\%HEAD\%/$html->folder("head")/se;
    $txt =~ s/\%BODY\%/$html->folder("body")/se;
    $txt =~ s/(\s+(?:href|src)=")([^"]+)/$1.$self->url(-mode => "loader", -name => $2)/sge;
    $html->append($txt);
    $html->divert("body");
    $html->overload(1);
    $self->param(-html => $html);

    #   establish database object
    my $dbi = DBI->connect(
        "dbi:SQLite:dbname=".$cfg->{-path_vardir}."/ase.db", "", "",
        { RaiseError => 1, AutoCommit => 1 }
    );
    die "failed to connect to database \"".$cfg->{-path_vardir}."/ase.db"."\""
        if (not defined($dbi));
    $self->param(-dbi => $dbi);

    #   establish session object
    my $sid = ($cgi->cookie("sid") || undef);
    my $session = new CGI::Session(
        "driver:DB_File;serializer:Storable;id:UUID", $sid,
        { Directory => $cfg->{-path_vardir}, FileName => "ase.sdb" }
    );
    $session->expires("+1h");
    $self->param(-session => $session);

    #   configure run-time modes
    my $modes = {
        #   global modes
        'error'              => { -login => 0, -method => \&error                     },
        'scode'              => { -login => 0, -method => \&mode_scode_render         },
        'loader'             => { -login => 0, -method => \&mode_loader               },

        #   login modes
        'login'              => { -login => 0, -method => \&mode_login_screen         },
        'login-action'       => { -login => 0, -method => \&mode_login_action         },
        'logout'             => { -login => 1, -method => \&mode_logout_screen        },
        'enrollment'         => { -login => 0, -method => \&mode_enrollment_screen    },
        'enrollment-action'  => { -login => 0, -method => \&mode_enrollment_action    },
        'activation'         => { -login => 0, -method => \&mode_activation_screen    },
        'recovery'           => { -login => 0, -method => \&mode_recovery_screen      },
        'recovery-action'    => { -login => 0, -method => \&mode_recovery_action      },

        #   application modes
        'menu-overview'      => { -login => 1, -method => \&mode_menu_overview_screen },
        'menu-member'        => { -login => 1, -method => \&mode_menu_member_screen   },
        'menu-member-action' => { -login => 1, -method => \&mode_menu_member_action   },
        'menu-account'       => { -login => 1, -method => \&mode_menu_account_screen  },
        'menu-action'        => { -login => 1, -method => \&mode_menu_action_screen   },
        'menu-logout'        => { -login => 1, -method => \&mode_menu_logout_screen   },
    };
    $self->param(-modes => $modes);
    my %run_modes = ();
    foreach my $mode (keys(%{$modes})) {
        $run_modes{$mode} = $modes->{$mode}->{-method};
    }
    $self->run_modes(%run_modes);
    $self->start_mode('menu-overview');
    $self->error_mode('error');
    $self->mode_param(\&current_mode);

    return;
}

#   helper function for determining run-time mode
sub current_mode {
    my ($self) = @_;
    my $cgi     = $self->param(-cgi);
    my $session = $self->param(-session);

    #   determine the current run-time mode
    my $modes = $self->param(-modes);
    my $mode  = ($cgi->url_param("mode") || $cgi->param("mode"));

    #   check for an unknown or invalid mode
    if (   not defined($mode)
        or $mode eq ''
        or not defined($modes->{$mode})) {
        $mode = $self->start_mode();
    }

    #   if mode is login and user is already
    #   logged in, enforce the regular start mode
    if (    $mode eq "login"
        and $session->param("login")) {
        $mode = $self->start_mode();
    }

    #   if mode requires a login and the user is 
    #   still not logged in, enforce the login!
    if (    $modes->{$mode}->{-login}
        and not $session->param("login")) {
        $mode = "login";
    }

    return $mode;
}

#   application post-processing hook
sub cgiapp_postrun {
    my ($self, $htmlref) = @_;

    #   replace output with expanded HTML document if
    #   no (non-HTML) output was generated instead.
    if (not defined(${$htmlref}) or ${$htmlref} eq '') {
        my $html = $self->param(-html);
        $html->overload(0);
        $html->undivert(0);
        ${$htmlref} = $html->string();
    }

    #   set the session cookie
    my $session = $self->param(-session);
    my $cookie = new CGI::Cookie(
        -name    => 'sid',
        -value   => $session->id,
        -expires => '+1h'
    );
    $self->header_add(-cookie => [$cookie]);

    #   switch to unbuffered output
    $|++;

    return;
}

#   application shutdown hook
sub teardown {
    my ($self) = @_;

    #   close session
    my $session = $self->param(-session);
    $session->close();

    #   disconnect from database
    my $dbi = $self->param(-dbi);
    $dbi->disconnect();

    return;
}

#   run-mode switching
sub switchto ($$;@) {
    my ($self, $mode, @args) = @_;
    my $modes = $self->param(-modes);
    if (not defined($modes->{$mode})) {
        die "cannot switch to unknown mode \"$mode\"";
    }
    my $method = $modes->{$mode}->{-method};
    return $self->$method($self, @args);
}

#   application error handling hook
sub error {
    my ($self, $error) = @_;
    my $html = "ERROR:<br>".HTML::Entities::encode_entities($error)."\n";
    return $html;
}

#   helper function for redirecting to a new URL 
sub redirect {
    my ($self, $url) = @_;

    #   establish redirection HTTP headers
    $self->header_type('redirect');
    $self->header_props(-url => $url);

    #   establish fallback redirection HTML page
    my $html =
        "You are redirected to the following URL:<br>" .
        "<a href=\"$url\">".HTML::Entities::encode_entities($url)."</a>" .
        "If nothing happens within 10 seconds," .
        "please click the above URL manually.\n";

    return $html;
}

1;

