##
##  OSSP ase -- Affiliation Service Environment
##  Copyright (c) 2005 Ralf S. Engelschall <rse@engelschall.com>
##  Copyright (c) 2005 The OSSP Project <http://www.ossp.org/>
##
##  This file is part of OSSP ase, a service environment for managing
##  affiliations which can be found at http://www.ossp.org/pkg/tool/ase/.
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
##  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
##
##  ase.cgi.profile.pl: profile workflow
##

package ASE;
use base 'CGI::Application';
use strict;
use warnings;

#   render navigation tab
sub menu_tab {
    my $self    = shift;
    my $mode    = shift;
    my $html    = $self->param(-html);

    #   define the menu
    my $menu = [
        { -mode => "menu-overview", -name => "Overview" },
        { -mode => "menu-member",   -name => "Member"   },
        { -mode => "menu-account",  -name => "Account"  },
        { -mode => "menu-action",   -name => "Action"   },
        { -mode => "menu-logout",   -name => "Logout"   },
    ];

    #   render the menu
    $html .=
        "<ul class=\"tabs\">\n" .
        "  <li class=\"padding\">&nbsp;</li>\n";
    foreach my $entry (@{$menu}) {
        $html .= "  ";
        $html .= ($entry->{-mode} eq $mode ? "<li class=\"active\">": "<li>");
        $html .= "<a href=\"".$self->url(-mode => $entry->{-mode})."\">".$entry->{-name}."</a>";
        $html .= "</li>\n";
    }
    $html .=
        "  <li class=\"padding\">&nbsp;</li>\n" .
        "</ul>\n";
    $html .= "<div class=\"tabsframe\">\n";
    $html->fold("foo");
    $html .= "</div>\n";
    $html >> "foo";

    return;
}

#   application run-time mode processing: "menu-overview"
sub mode_menu_overview_screen {
    my $self    = shift;
    my $cgi     = $self->param(-cgi);
    my $dbi     = $self->param(-dbi);
    my $html    = $self->param(-html);
    my $session = $self->param(-session);

    #   generate the outer canvas
    $html .= "<div class=\"boxed\">\n";
    $html .= "<span class=\"title\">Main Menu</span>\n";
    $html .= $cgi->start_form(
        -method => 'POST',
        -action => $self->url(),
        -class  => 'menu',
    ) . "\n";
    $html .= $cgi->hidden(
        -name     => 'mode',
        -value    => 'menu-action',
        -override => 1
    ) . "\n";
    $html->fold("canvas");
    $html .= $cgi->end_form() . "\n";
    $html .= "</div>\n";
    $html >> "canvas";

    #   insert navigation tab
    $self->menu_tab("menu-overview");

    #   take membership UUID from session
    my $me_uuid = $session->param("login");

    #   fetch remaining member details from database
    my $sth = $dbi->prepare(
        "SELECT * FROM ase_member WHERE me_uuid = ?;")
        or die $dbi->errstr();
    $sth->execute($me_uuid)
        or die $dbi->errstr();
    my $rec = $sth->fetchrow_hashref()
        or die "no member found under UUID $me_uuid";

    #   determine name of membership status
    my $me_status = (qw(Enrolling Fellow Member Activist Director President))
        [$rec->{'me_membership_status'} || 0];

    $html .=
        "<table class=\"F\">\n" .
        "  <tr>\n" .
        "    <td class=\"L\"><span class=\"label\">Status:</span></td>\n" .
        "    <td class=\"I\">". $me_status ."</td>\n" .
        "  </tr>\n" .
        "  <tr>\n" .
        "    <td class=\"L\"><span class=\"label\">UUID:</span></td>\n" .
        "    <td class=\"I\">". $me_uuid ."</td>\n" .
        "  </tr>\n" .
        "  <tr>\n" .
        "    <td class=\"L\"><span class=\"label\">Email Address:</span></td>\n" .
        "    <td class=\"I\">". $rec->{"me_address"} ."</td>\n" .
        "  </tr>\n" .
        "  <tr>\n" .
        "    <td class=\"L\"><span class=\"label\">Membership Start:</span></td>\n" .
        "    <td class=\"I\">". $self->datetime2str($rec->{'me_membership_start'}) ."</td>\n" .
        "  </tr>\n" .
        "  <tr>\n" .
        "    <td class=\"L\"><span class=\"label\">Membership End:</span></td>\n" .
        "    <td class=\"I\">". $self->datetime2str($rec->{'me_membership_end'}) ."</td>\n" .
        "  </tr>\n" .
        "</table>\n";

    return;
}

#   application run-time mode processing: "menu-member"
sub mode_menu_member_screen {
    my $self    = shift;
    my $cgi     = $self->param(-cgi);
    my $dbi     = $self->param(-dbi);
    my $html    = $self->param(-html);
    my $session = $self->param(-session);

    #   generate the outer canvas
    $html .= "<div class=\"boxed\">\n";
    $html .= "<span class=\"title\">Main Menu</span>\n";
    $html .= $cgi->start_form(
        -method => 'POST',
        -action => $self->url(),
        -class  => 'menu',
    ) . "\n";
    $html .= $cgi->hidden(
        -name     => 'mode',
        -value    => 'menu-member-action',
        -override => 1
    ) . "\n";
    $html->fold("canvas");
    $html .= $cgi->end_form() . "\n";
    $html .= "</div>\n";
    $html >> "canvas";

    #   insert navigation tab
    $self->menu_tab("menu-member");

    #   take membership UUID from session
    my $me_uuid = $session->param("login");

    #   fetch remaining member details from database
    my $sth = $dbi->prepare(
        "SELECT * FROM ase_member WHERE me_uuid = ?;")
        or die $dbi->errstr();
    $sth->execute($me_uuid)
        or die $dbi->errstr();
    my $rec = $sth->fetchrow_hashref()
        or die "no member found under UUID $me_uuid";

    #   determine name of membership status
    my $me_status = (qw(Enrolling Fellow Member Activist Director President))
        [$rec->{'me_membership_status'} || 0];

    $html .= "<table class=\"F\">\n";
    $html .=
        "  <tr>\n" .
        "    <td class=\"L\"><span class=\"label\">"."foo".":</span></td>\n" .
        "    <td class=\"I\">"."foo"."</td>\n" .
        "  </tr>\n";
    $html .= "</table>\n";

    #   render member edit form
    my $form = new CGI::FormBuilder (
        "fields"   => [qw(me_sn me_uuid me_username me_address me_membership_status me_membership_prop me_gender)],
        "method"   => "POST",
        "values"   => $rec,
        "header"   => 0,
        "action"   => $self->url(),
        "class"    => "menu",
        "template" => &render,
    );
    $form->field(
        "name"     => "me_membership_status",
        "label"    => "Membership Status",
        "options"  => [
            [ "0" => "enrolling" ],
            [ "1" => "fellow"    ],
            [ "2" => "member"    ],
            [ "3" => "activist"  ],
            [ "4" => "director"  ],
            [ "5" => "president" ],
        ],
    );
    sub render {
        my ($form) = @_;
    }
    $html .= $form->render();

    return;
}

#   run-time mode screen action: "menu-member-action"
sub mode_menu_member_action {
    my $self    = shift;
    my $cgi     = $self->param(-cgi);
    my $dbi     = $self->param(-dbi);
    my $session = $self->param(-session);

    return "FUCK";
}

1;

