##
##  OSSP ase -- Affiliation Service Environment
##  Copyright (c) 2005 Ralf S. Engelschall <rse@engelschall.com>
##  Copyright (c) 2005 The OSSP Project <http://www.ossp.org/>
##
##  This file is part of OSSP ase, a service environment for managing
##  affiliations which can be found at http://www.ossp.org/pkg/tool/ase/.
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
##  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
##
##  ase.cgi.scode.pl: security code handling
##

package ASE;
use base 'CGI::Application';
use strict;
use warnings;

#   helper function for converting hex "RRGGBB" into decimal (R,G,B)
sub hex2rgb {
    my ($hex) = @_;
    return map { hex($_) }
        ($hex =~ m/^([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/);
}

#   Security Code: creation
sub scode_create {
    my ($self) = @_;
    my $dbi = $self->param(-dbi);

    #   generate new UUID
    my $uuid = new OSSP::uuid;
    $uuid->make("v1");
    my $scode_uuid = $uuid->export("str");
    undef $uuid;

    #   generate new Security Code Number
    #   (notice: "0" and "O" are too hard to distinguish, so left out)
    my $scode_num = $self->makerandom(6, "1-9A-Z");

    #   determine expiry time
    my $scode_expiry = time() + 4*60*60;

    #   on-the-fly remove expired codes
    my $now = time();
    $dbi->do(
        "DELETE FROM ase_scode WHERE sc_expiry < $now;"
    ) or die $dbi->errmsg;;

    #   store Security Code in database
    $dbi->do(
        "INSERT INTO ase_scode (sc_uuid, sc_number, sc_expiry)" .
        " VALUES ('$scode_uuid', '$scode_num', $scode_expiry);"
    ) or die $dbi->errmsg;;
    
    return $scode_uuid;
}

#   Security Code: image rendering
sub scode_render {
    my ($self, $scode_uuid) = @_;
    my $dbi = $self->param(-dbi);

    #   determine Security Code Number
    my $rec = $dbi->selectrow_hashref(
        "SELECT sc_number FROM ase_scode" .
        " WHERE sc_uuid = '$scode_uuid';"
    ) or die $dbi->errstr();
    my $scode_num = $rec->{'sc_number'};

    #   render Security Code Number as a PNG image
    my $im_length = (length($scode_num)+1)*10;
    my $im = new GD::Image($im_length, 25);
    my $c_bg  = $im->colorAllocate(&hex2rgb("ffffff"));
    my $c_bd1 = $im->colorAllocate(&hex2rgb("999999"));
    my $c_bd2 = $im->colorAllocate(&hex2rgb("cccccc"));
    my $c_ln  = $im->colorAllocate(&hex2rgb("dddddd"));
    my $c_tx  = $im->colorAllocate(&hex2rgb("888888"));
    $im->fill(0, 0, $c_bg);
    for (my $i = 5; $i < $im_length; $i += 5) {
        $im->line($i, 0, $i, 24, $c_ln);
    }
    for (my $i = 5; $i < 25; $i += 5) {
        $im->line(0, $i, $im_length-1, $i, $c_ln);
    }
    $im->line(0, 0, $im_length-1, 0, $c_bd1);
    $im->line(0, 0, 0, 24, $c_bd1);
    $im->line($im_length-1, 0, $im_length-1, 24, $c_bd2);
    $im->line(0, 24, $im_length-1, 24, $c_bd2);
    $im->string(GD::gdGiantFont(), 8, 5, $scode_num, $c_tx);
    my $scode_img = $im->png;

    return $scode_img;
}

#   Security Code: verification
sub scode_verify {
    my ($self, $scode_uuid, $scode_num) = @_;
    my $dbi = $self->param(-dbi);

    #   determine Security Code
    my $rec = $dbi->selectrow_hashref(
        "SELECT sc_number, sc_expiry FROM ase_scode" .
        " WHERE sc_uuid = '$scode_uuid'"
    ) or die $dbi->errstr();
    my $scode_number = $rec->{'sc_number'};
    my $scode_expiry = $rec->{'sc_expiry'};

    #   verify Security Code
    my $ok = ($scode_num eq $scode_number and time() < $scode_expiry);

    return $ok;
}

#   run-time mode image rendering: "scode"
sub mode_scode_render {
    my $self = shift;
    my $cgi  = $self->param(-cgi);

    #   send Security Code image
    my $uuid = $cgi->param("uuid");
    my $img = $self->scode_render($uuid);
    $self->header_props(-type => 'image/png', -expires => '+4d');
    return $img;
}

1;

