--
--  OSSP ase -- Affiliation Service Environment
--  Copyright (c) 2005 Ralf S. Engelschall <rse@engelschall.com>
--  Copyright (c) 2005 The OSSP Project <http://www.ossp.org/>
--
--  This file is part of OSSP ase, a service environment for managing
--  affiliations which can be found at http://www.ossp.org/pkg/tool/ase/.
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 2 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
--  General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
--  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
--
--  ase.sql: OSSP ase database schema
--

--  Member Information
CREATE TABLE ase_member (
    -- Unique Key (primary)
    me_sn                      INTEGER,
                               -- Membership Sequence/Serial Number
                               -- [7]
    me_uuid                    TEXT UNIQUE NOT NULL PRIMARY KEY, 
                               -- Membership UUID
                               -- [75468374-8721-11d9-bee4-0002a5489f14]

    -- Unique Keys (secondary)
    me_username                TEXT UNIQUE,
                               -- Username
                               -- [rse]
    me_address                 TEXT UNIQUE,
                               -- Email-Address
                               -- [rse@engelschall.com]

    -- Membership Details
    me_membership_status       CHAR(1) NOT NULL,
                               -- Membership Status/Karma: 0=enrolling, 1=fellow, 2=member, 3=activist, 4=director, 5=president
                               -- [5]
    me_membership_type         CHAR(1) NOT NULL,
                               -- Membership Type: N=natural, J=juristic
                               -- [N]
    me_membership_class        CHAR(1),
                               -- Membership Class: R=regular, S=sustaining
                               -- [R]
    me_membership_prop         CHAR(4),
                               -- Membership Properties: A=admin, F=founding, H=honorary, ...
                               -- [F]
    me_membership_start        INTEGER,
                               -- Membership Start Date
                               -- [20050209]
    me_membership_end          INTEGER,
                               -- Membership End Date
                               -- [20060209]

    -- Person Details
    me_gender                  CHAR(1),
                               -- Gender: M=male, F=female
                               -- [M] 
    me_name_title              TEXT,
                               -- Title
                               -- [Dipl.-Inf.]
    me_name_first              TEXT,
                               -- First Name
                               -- [Ralf]
    me_name_middle             CHAR(1),
                               -- Middle Initial
                               -- [S]
    me_name_last               TEXT,
                               -- Last/Family Name
                               -- [Engelschall]
    me_birthday                INTEGER,
                               -- Birthday
                               -- [19721117]
    me_languages               TEXT,
                               -- Languages ISO-639 Codes (primary[,secondary,...])
                               -- [DE,EN]

    -- Person Description
    me_profession              TEXT,
                               -- Profession
                               -- [Computer Scientist]
    me_employer                TEXT,
                               -- Currenty Employer
                               -- [Cable & Wireless]
    me_biography               TEXT,
                               -- One-Paragraph Biography Text
                               -- [...]
    me_portrait                BLOB,
                               -- Portrait Picture (PNG or JPEG format)
                               -- [...]
    me_homepage                TEXT,
                               -- Internet/WWW Homepage
                               -- [http://www.engelschall.com/]
    me_geekcode                TEXT,
                               -- GeekCode (see http://www.geekcode.com/)
                               -- [...]
    me_usc                     TEXT,
                               -- Unified Skill Code (RSE's USC)
                               -- [...]
    me_openpkg_since           INTEGER,
                               -- OpenPKG Known to you since...
                               -- [20001115]

    -- Bank Account
    me_bank_institute_iso3166  CHAR(2),
                               -- Bank, Institute ISO 3166 Country Code
                               -- [DE]
    me_bank_institute_name     TEXT,
                               -- Bank, Institute Name
                               -- [Deutsche Bank]
    me_bank_institute_number   TEXT,
                               -- Bank, Institute Number
                               -- [70070024]
    me_bank_account_number     TEXT,
                               -- Bank, Account Number
                               -- [7700735]
    me_bank_directdebiting     INTEGER,
                               -- Bank, Direct Debiting: 0=no, 1=yes
                               -- [1]

    -- Person Location
    me_loc_postal_street       TEXT,
                               -- Postal Location, Street
                               -- [Silnerstr.]
    me_loc_postal_house        TEXT,
                               -- Postal Location, House Number
                               -- [28]
    me_loc_postal_zipcode      TEXT,
                               -- Postal Location, Zip Code
                               -- [85221]
    me_loc_postal_city         TEXT,
                               -- Postal Location, City
                               -- [Dachau]
    me_loc_postal_country      TEXT,
                               -- Postal Location, Country
                               -- [Germany]
    me_loc_postal_iso3166      TEXT,
                               -- Postal Location, ISO-3166-1 Country Code
                               -- [DE]
    me_loc_timezoneoffset      INTEGER,
                               -- Positional Location, Timezone Offset (-12, ..., +12) without DST
                               -- [+1]
    me_loc_longitude           REAL,
                               -- Positional Location, Longitude
                               -- [11.43333]
    me_loc_latidude            REAL,
                               -- Positional Location, Lattitude
                               -- [48.26667]

    -- Access
    me_access_pw_sha1          TEXT NOT NULL,
                               -- Access, Password, plain SHA1
                               -- [HHHHHHHH...]
    me_access_pw_md5           TEXT NOT NULL,
                               -- Access, Password, plain MD5
                               -- [HHHHHHHH...]
    me_access_pw_cryptmd5      TEXT NOT NULL,
                               -- Access, Password, crypt(3) salted MD5
                               -- [$1$5OnRPADJ$NOwJzToVtRRFngRxTM21g/]
    me_access_pw_cryptdes      TEXT NOT NULL,
                               -- Access, Password, crypt(3) salted DES
                               -- [xxxxxxxxxx]
    me_access_pk_ssh2          TEXT,
                               -- Access, Public Key, SSH-2 RSA/DSA (SSH, ssh-keygen)
                               -- [........]
    me_access_pk_openpgp       TEXT,
                               -- Access, Public Key, OpenPGP (PGP, GPG, gpg)
                               -- [........]
    me_access_pk_x509          TEXT,
                               -- Access, Public Key, X.509 Certificate (SSL, S/MIME, openssl)
                               -- [........]
    
    -- Account Recovery
    me_recovery_secret         TEXT,
                               -- Recovery Alternative 1: Secret
                               -- [...]
    me_recovery_question       INTEGER,
                               -- Recovery Alternative 2: Question: 0=none, 1=mother-maiden-name, 2=father-city-of-birth, 3=last-4-chars-of-creditcard 4=last-4-chars-of-idcard
                               -- [1]
    me_recovery_answer         TEXT
                               -- Recovery Alternative 2: Answer: 0=none, 1=mother-maiden-name, 2=father-city-of-birth, 3=last-4-chars-of-creditcard 4=last-4-chars-of-idcard
                               -- [1]
);

CREATE TABLE ase_enrollment (
    en_uuid                    TEXT UNIQUE NOT NULL PRIMARY KEY, 
                               -- Membership UUID
                               -- [75468374-8721-11d9-bee4-0002a5489f14]
    en_address                 TEXT UNIQUE NOT NULL,
                               -- Email-Address
                               -- [rse@engelschall.com]
    en_pw_sha1                 TEXT NOT NULL,
                               -- Access, Password, plain SHA1
                               -- [HHHHHHHH...]
    en_pw_md5                  TEXT NOT NULL,
                               -- Access, Password, plain MD5
                               -- [HHHHHHHH...]
    en_pw_cryptmd5             TEXT NOT NULL,
                               -- Access, Password, crypt(3) salted MD5
                               -- [$1$5OnRPADJ$NOwJzToVtRRFngRxTM21g/]
    en_pw_cryptdes             TEXT NOT NULL,
                               -- Access, Password, crypt(3) salted DES
                               -- [xxxxxxxxxx]
    en_start                   INTEGER NOT NULL,
                               -- Enrollment Start (generated, YYYYMMDDHHMMSS)
                               -- [20050225000000]
    en_expiry                  INTEGER NOT NULL
                               -- Enrollment Expiry (generated, YYYYMMDDHHMMSS)
                               -- [20050225000000]
);

--  Action Logbook
CREATE TABLE ase_logbook (
    -- Unique Key (primary)
    lg_uuid                    TEXT, 
                               -- Membership UUID
                               -- [75468374-8721-11d9-bee4-0002a5489f14]

    -- Log Entry
    lg_date                    INTEGER NOT NULL,
                               -- Log Entry Date YYYYMMDDHHMMSS
                               -- [20050225000000]
    lg_text                    TEXT NOT NULL
                               -- Log Entry Text
                               -- [...]
);

--  Financial Accounts
CREATE TABLE ase_account (
    -- Unique Key
    ac_uuid                    TEXT NOT NULL, 
                               -- Membership UUID
                               -- [75468374-8721-11d9-bee4-0002a5489f14]

    -- Transaction Entry
    ac_date                    INTEGER NOT NULL,
                               -- Account Transaction Date YYYYMMDDHHMMSS
                               -- [20050225000000]
    ac_purpose                 TEXT NOT NULL,
                               -- Account Transaction Purpose
                               -- [Membership 2005/02]
    ac_amount                  REAL NOT NULL
                               -- Account Transaction Amount +/-N
                               -- [+123.50]
);

--  Temporary Security Codes
CREATE TABLE ase_scode (
    sc_uuid                    TEXT NOT NULL,
                               -- Scode UUID
                               -- [75468374-8721-11d9-bee4-0002a5489f14]
    sc_number                  TEXT NOT NULL,
                               -- Scode Number (4 chars)
                               -- [xxxx]
    sc_expiry                  INTEGER NOT NULL
                               -- Scode Expiry Date/Time YYYYMMDDHHMMSS
                               -- [20050225000000]
);

